package com.jcommercesql.gateway.authorizenet;

/**
 * This is part of JCommerceSQL Authorize.net Gateway
 *
 * <b>Copyright 2003 by Richard Roehl. All Rights Reserved.</b>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.util.Enumeration;
import java.io.IOException;

/**
 *  AuthorizeNetCC can be used to perform credit card authorizations through Authorize.net. 
 *  <p>
 *  <b>Basic Example 1:</b>
 * <pre><code>
 *  AuthorizeNetCC anetcc = new AuthorizeNetCC();
 *  anetcc.setURL("secure.authorize.net",443,"/gateway/transact.dll");
 *  anetcc.setMerchantInfo("loginid","transactionkey");
 *  anetcc.addOptionalField("x_Address","123 Main St.");	// if needed for AVS
 *  anetcc.addOptionalField("x_Zip","10101");			// if needed for AVS
 *  anetcc.addMerchantDefinedField("m_Comment", "Basic Example 1");
 *  anetcc.setTransaction("1234123412341234","11/09","100.00","AUTH_CAPTURE");
 *  try {
 *    anetcc.submit();
 *    if (anetcc.getResponseCode().equals("1")) {
 *	System.out.println("Your transaction was approved!");
 *    } else {
 *	System.out.println("Your transaction was declined! " + anetcc.getResponseReasonText());
 *    }
 *  } catch(Exception e) {
 *      System.out.println("System Error! " + e);
 *  }  
 * </code></pre>
 *
 *  <p>
 *  <b>Basic Example 2:</b>
 * <pre><code>
 *  AuthorizeNetCC anetcc = new AuthorizeNetCC("/path/to/config.prop");
 *  anetcc.addOptionalField("x_Address","123 Main St.");	// if needed for AVS
 *  anetcc.addOptionalField("x_Zip","10101");			// if needed for AVS
 *  anetcc.addMerchantDefinedField("m_Comment", "Basic Example 2");
 *  anetcc.setTransaction("1234123412341234","11/09","100.00","AUTH_CAPTURE");
 *  try {
 *    anetcc.submit();
 *    if (anetcc.getResponseCode().equals("1")) {
 *	System.out.println("Your transaction was approved!");
 *    } else {
 *	System.out.println("Your transaction was declined! " + anetcc.getResponseReasonText());
 *    }
 *  } catch(Exception e) {
 *      System.out.println("System Error! " + e);
 *  }  
 * </code></pre>
 *
 * This class was designed using the Authorize.net Advanced Integration Method Implementation Guide v1.0.
 * It is recommended that you download a copy of this guide to fully understand what gateway methods should
 * be used for your particular situation.
 * <p>
 *
 */

public final class AuthorizeNetCC extends AuthorizeNet {

  //int AUTH_CAPTURE=0;
  //int AUTH_ONLY=1;

  //int[] TRANS_TYPES={AUTH_CAPTURE,AUTH_ONLY,CAPTURE_ONLY,CREDIT,VOID,PRIOR_AUTH_CAPTURE}
  //protected URL="";
  // Common to both Credit Card and eCheck transactions
  //x_Version="";
  //x_Delim_Data="";
  //x_Login="";
  //x_Tran_Key="";
  //x_Amount="";

  // Minimum information required for credit card = common fields above + following 3 fields
  protected String x_Card_Num="";
  protected String x_Exp_Date="";
  protected String x_Type="";

  /**
    * Initializing Constructor
    *
    */
  public AuthorizeNetCC() {};

  /**
    * Initializing Constructor.  
    * @param filename Name of properties file containing Authorize.Net host, port, path, login ID and transaction key
    *
    */
  public AuthorizeNetCC( String filename ) throws IOException, InvalidPropException {
    this.properties=loadPropFile(filename);
    this.propFileName=filename;
  };

//---------------------------------------------------------------------------- 
/*  public static void main(String[] args) {

    System.out.println("Going in");

    //String postdata="x_Version=3.1&x_Delim_Data=True&x_Login=test&x_Tran_Key=123&x_Amount=5.99&x_Card_Num=370000000000002&x_Exp_Date=11/09&x_Type=AUTH_CAPTURE&x_Test_Request=TRUE&x_Delim_Char=.&x_Encap_Char=\"";

    AuthorizeNetCC a = new AuthorizeNetCC();
    a.setTransaction("1234123412341234","11/04","25.99","AUTH_CAPTURE");
    a.setHost("secure.authorize.net");
    a.setPort(443);
    a.setPath("/gateway/transact.dll");
    a.setLogin("/gateway/transact.dll");
    a.setTranKey("/gateway/transact.dll");
try {
    a.submit();
} catch(Exception e) {System.out.println("Submit Exception: " +e); }
    System.out.println("Going out");

    System.out.println("Here is postdata: " + a.getPostData());
    System.out.println("Here is raw data: "+ new String(a.rawResult));
     
  } 
*/

//---------------------------------------------------------------------------- 
  /**
    * Set Credit Card Number
    * @param cardNum Credit Card Number
    */
  public void setCardNum(String cardNum) { this.x_Card_Num=cardNum; }

//---------------------------------------------------------------------------- 
  /**
    * Returns the Credit Card Number the was set by the <code>setCreditCard()</code> method.
    *
    */
  public String getCardNum() { return x_Card_Num; }

//---------------------------------------------------------------------------- 
  /**
    * Set Credit Card Expiration Date.
    * @param expDate Credit Card Expiration Date
    * @see AuthorizeNetCC#setTransaction(String cardNum, String expDate, String amount, String transType) setTransaction()
    * 
    */
  public void setExpDate(String expDate) { this.x_Exp_Date=expDate; }

//---------------------------------------------------------------------------- 
  /**
    * Returns Credit Card Expiration Date as set by the <code>setExpDate()</code> method.
    *
    */
  public String getExpDate() { return x_Exp_Date; }

//---------------------------------------------------------------------------- 
  /**
    * Set the Credit Card Transaction Type. 
    *
    * Example Types are: AUTH_CAPTURE, AUTH_ONLY, CAPTURE_ONLY, CREDIT, VOID, PRIOR_AUTH_CAPTURE.<p>
    * See the Authorize.net Advanced Implementation Guide for the correct values for your transaction.
    *
    * @param transType Credit Card Transaction Type
    * @see <a href="http://authorize.net">Authorize.net</a>
    */
  public void setType(String transType) { this.x_Type=transType; }

//---------------------------------------------------------------------------- 
  /**
    * Returns the Credit Card Transaction Type as set by the <code>setType()</code> method.
    *
    */
  public String getType() { return x_Type; }

//---------------------------------------------------------------------------- 
  /**
    * Set Transaction Information.
    *
    * This method is provided as an alternative to setting the items with individual calls.
    *
    * @param cardNum Credit Card Number
    * @param expDate Credit Card Expiration Date
    * @param amount  Amount of Transaction
    * @param transType Transaction Type
    *
    * @see AuthorizeNetCC#setType(String transType) setType
    */
  public void setTransaction(String cardNum, String expDate, String amount, String transType) { 

    setCardNum(cardNum);
    setExpDate(expDate);
    setAmount(amount);
    setType(transType);
  }

//---------------------------------------------------------------------------- 
  /**
    *
    *
    */
  private boolean verifyMinCCFieldsSet() { 

    boolean result=true;
    if (x_Card_Num==null)  result = false;
    if (x_Exp_Date==null || x_Delim_Data.equals("") ) result = false;
    if (x_Type==null || x_Type.equals("") ) result = false;
    return result;
  }

//---------------------------------------------------------------------------- 
  /**
    * Show Post Data.
    *
    * The Authorize.net transaction gateway expects a list of parameters and values to be
    * posted to their site.  Use this function to print out the the list for troubleshooting
    * purposes.  Note: this will only return data once the submit function has been called.
    *
    */
  public String getPostData() { 
    return postData;
  }
//---------------------------------------------------------------------------- 
  /**
    * Submit Credit Card Transaction For Approval.
    *
    * A RequiredFieldException is thrown if the following fields are not set:
    * credit card number, expiration date, amount, transaction type
    * authorize.net server name, port, path,
    * merchant account login and transaction key.
    */
  public void submit() throws RequiredFieldException,Exception { 

    String host=getHost();
    String path=getPath();
    int port=getPort();
    String postdata;

    if (!verifyMinCCFieldsSet()) {
      throw new RequiredFieldException("One of the following fields is not set: Card_Num,Exp_Date,Type");
    }

    if (!verifyMinimumFieldsSet()) {
      throw new RequiredFieldException("One of the following fields is not set: login, transaction key, amount");
    }

    if ( host==null || host.equals("") ) {
	throw new RequiredFieldException("Host Field Not Set");
    }
    if ( path==null || path.equals("") ) {
	throw new RequiredFieldException("URL Path Field Not Set");
    }

	// add requiredfields to post data
    postdata=AuthorizeNetCodes.REQ_FIELD_VERSION+"="+getVersion();
    postdata+="&"+AuthorizeNetCodes.REQ_FIELD_LOGIN+"="+getLogin();
    postdata+="&"+AuthorizeNetCodes.REQ_FIELD_TRAN_KEY+"="+getTranKey();
    postdata+="&"+AuthorizeNetCodes.REQ_FIELD_AMOUNT+"="+getAmount();
    postdata+="&"+AuthorizeNetCodes.REQ_FIELD_CARD_NUM+"="+getCardNum();
    postdata+="&"+AuthorizeNetCodes.REQ_FIELD_EXP_DATE+"="+getExpDate();
    postdata+="&"+AuthorizeNetCodes.REQ_FIELD_TYPE+"="+getType();
    postdata+="&"+AuthorizeNetCodes.REQ_FIELD_DELIM_DATA+"="+getDelimData();
    postdata+="&"+AuthorizeNetCodes.REQ_FIELD_DELIM_CHAR+"="+getDelimChar();

	// add optional fields to post data
    Enumeration e = optionalFields.keys();
    while (e.hasMoreElements()) {
	  String field = (String)e.nextElement();
	  postdata+="&"+field+"="+(String)optionalFields.get(field);
    }

	// add merchant defined fields to post data
	int idx=0;    
	while (merchantDefinedFields.size() > idx) {
		postdata+="&"+(String)merchantDefinedFields.get(idx)+"="+(String)merchantDefinedFields.get(idx+1);
		idx+=2;    
	}

	 this.postData=postdata;

    submitANetPost(host,port,path,postdata);
 
  } // end submit()
}
